@@ -24,6 +24,7 @@
 # Unless otherwise stated in the comments, "id", in e.g. user_id, refers to the
 # internal Keycloak server ID, usually a uuid string
 
+import copy
 import json
 
 from .connection import ConnectionManager
@@ -37,8 +38,18 @@ from .urls_patterns import URL_ADMIN_SERVER_INFO, URL_ADMIN_CLIENT_AUTHZ_RESOURC
     URL_ADMIN_REALM_IMPORT, URL_ADMIN_USERS_COUNT, URL_ADMIN_FLOWS, URL_ADMIN_GROUP, URL_ADMIN_CLIENT_AUTHZ_SETTINGS, \
     URL_ADMIN_GROUP_MEMBERS, URL_ADMIN_USER_STORAGE, URL_ADMIN_GROUP_PERMISSIONS, URL_ADMIN_IDPS, \
     URL_ADMIN_USER_CLIENT_ROLES_AVAILABLE, URL_ADMIN_USERS, URL_ADMIN_CLIENT_AUTHZ_RESOURCES_ID, URL_ADMIN_CLIENT_AUTHZ_POLICIES, \
-    URL_ADMIN_CLIENT_AUTHZ_POLICIES_ID, URL_ADMIN_CLIENT_AUTHZ_PERMISSIONS, URL_ADMIN_CLIENT_AUTHZ_PERMISSIONS_ID
+    URL_ADMIN_CLIENT_AUTHZ_POLICIES_ID, URL_ADMIN_CLIENT_AUTHZ_PERMISSIONS, URL_ADMIN_CLIENT_AUTHZ_PERMISSIONS_ID, \
+    URL_ADMIN_USER_CLIENT_ROLES_COMPOSITE_ID
 
+def serialize(obj):
+    """JSON serializer for objects not serializable by default json code"""
+    _obj = copy.copy(obj)
+
+    for key in _obj.__dict__:
+        if type(_obj.__dict__[key]) is bool:
+            _obj.__dict__[key] = str(_obj.__dict__[key]).lower()
+
+    return _obj.__dict__
 
 class KeycloakAdmin:
 
@@ -614,7 +625,7 @@ class KeycloakAdmin:
         """
         params_path = {"realm-name": self.realm_name, "id": client_id}
         data_raw = self.connection.raw_post(URL_ADMIN_CLIENT_AUTHZ_RESOURCES.format(**params_path),
-                                            data=json.dumps(payload))
+                                            data=json.dumps(payload, default=serialize))
         return raise_error_from_response(data_raw, KeycloakGetError, expected_code=201)
 
     def update_client_authz_resource(self, client_id, resource_id, payload):
@@ -836,21 +847,21 @@ class KeycloakAdmin:
         role = self.get_client_role(client_id, role_name)
         return role.get("id")
 
-    def create_client_role(self, client_role_id, payload, skip_exists=False):
+    def create_client_role(self, client_id, payload, skip_exists=False):
         """
         Create a client role
 
         RoleRepresentation
         http://www.keycloak.org/docs-api/3.3/rest-api/index.html#_rolerepresentation
 
-        :param client_role_id: id of client (not client-id)
+        :param client_id: id of client (not client-id)
         :param payload: RoleRepresentation
         :return: Keycloak server response (RoleRepresentation)
         """
 
-        params_path = {"realm-name": self.realm_name, "id": client_role_id}
+        params_path = {"realm-name": self.realm_name, "id": client_id}
         data_raw = self.connection.raw_post(URL_ADMIN_CLIENT_ROLES.format(**params_path),
-                                            data=json.dumps(payload))
+                                           data=json.dumps(payload, default=serialize))
         return raise_error_from_response(data_raw, KeycloakGetError, expected_code=201, skip_exists=skip_exists)
 
     def delete_client_role(self, client_role_id, role_name):
@@ -867,13 +878,12 @@ class KeycloakAdmin:
         data_raw = self.connection.raw_delete(URL_ADMIN_CLIENT_ROLE.format(**params_path))
         return raise_error_from_response(data_raw, KeycloakGetError, expected_code=204)
 
-    def assign_client_role(self, user_id, client_id, roles):
+    def assign_client_role(self, client_id, user_id, roles):
         """
         Assign a client role to a user
 
-        :param client_id: id of client (not client-id)
-        :param user_id: id of user
         :param client_id: id of client containing role,
+        :param user_id: id of user
         :param roles: roles list or role (use RoleRepresentation)
         :return Keycloak server response
         """
@@ -914,6 +924,12 @@ class KeycloakAdmin:
         """
         return self._get_client_roles_of_user(URL_ADMIN_USER_CLIENT_ROLES_COMPOSITE, user_id, client_id)
 
+    def create_composite_client_role(self, role_id, payload, skip_exists=False):
+        params_path = {"realm-name": self.realm_name, "role-id": role_id}
+        data_raw = self.connection.raw_post(URL_ADMIN_USER_CLIENT_ROLES_COMPOSITE_ID.format(**params_path),
+                                            data=json.dumps(payload, default=serialize))
+        return raise_error_from_response(data_raw, KeycloakGetError, expected_code=204, skip_exists=skip_exists)
+
     def _get_client_roles_of_user(self, client_level_role_mapping_url, user_id, client_id):
         params_path = {"realm-name": self.realm_name, "id": user_id, "client-id": client_id}
         data_raw = self.connection.raw_get(client_level_role_mapping_url.format(**params_path))
